--  ===============================
--	Faction Addict - FactionAddict.lua
--	By: gmz323(Greg)
--  ===============================

-- Constants
local FACTION_ADDICT_VERSION = "1.20";
local FACTION_ADDICT_LOGGING_VERSION = 1;
local FACTION_ADDICT_LOGGING_DAYS = 20;
local GUILD_FACTION_ID = 1168;
-- Locals
local tempDisplayData = {} -- temp table built to show subset of data
local tempDisplayData_rowcount;
local tempAllFactionData = {} -- table built to store all faction data - info window population
local tempAllFactionData_rowcount;
local tempLoggingData = {} -- temp table to show log data
local tempLoggingData_rowcount;
local tempFactionIDsByName = {} -- temp table mapping factin names to IDs
local tempAchievementTooltipData = {} -- table holding achievement tooltip data
local doServerFactionQuery = 0;
local faUpdateDisplayTable;
local faNumFactions;
local faNumFactionsExalted;
local faClickedFactionName; -- holds clicked faction name for info window
local faLastRepGainStr;
local faSessionRepGain;
local faSessionDate;
local faStartTime = GetTime();
local faPlayerGuildName;
local lfgBonusFactionID;

-- Phanx vars
local running;
local isInactive = {};
local isInReputationDisplay = {};

-- Debugging
local debugMode = false;

-- Localized Strings
local L = FactionAddictLocalization;

-- config table defaults - saved per character
local FactionAddictConfigDefaults = 
	{
		CB_STANDING_EXALTED = true,
		CB_STANDING_REVERED = true,
		CB_STANDING_HONORED = true,
		CB_STANDING_FRIENDLY = true,
		CB_STANDING_NEUTRAL1 = true,
		CB_STANDING_NEUTRAL0 = true,
		CB_STANDING_UNFRIENDLY = true,
		CB_STANDING_HOSTILE = true,
		CB_STANDING_HATED = true,
		CB_CATEGORY_CF = true,
		CB_CATEGORY_BC = true,
		CB_CATEGORY_WOTLK = true,
		CB_CATEGORY_CATA = true,
		CB_CATEGORY_MOP = true,
		CB_CATEGORY_WOD = true,
		RB_ORDERBY = 1,
		CB_FILTER_SHOW_AO = true,
		CB_FILTER_SHOW_HO = true,
		CB_OPTION_AUTOBAR = true,
		CB_OPTION_HIDEINACTIVE = true,
	};
	
local faGender = UnitSex("player");
local ConfigMenuInfo = {
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."8", faGender) .. " (42,000+)", ["arg1"] = "CB_STANDING_EXALTED",  },
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."7", faGender) .. " (21,000 to 42,000)", ["arg1"] = "CB_STANDING_REVERED",  },
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."6", faGender) .. " (9,000 to 21,000)", ["arg1"] = "CB_STANDING_HONORED",  },
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."5", faGender) .. " (3,000 to 9,000)", ["arg1"] = "CB_STANDING_FRIENDLY", },
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."4", faGender) .. " (1 to 3,000)", ["arg1"] = "CB_STANDING_NEUTRAL1", },
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."4", faGender) .. " (=0)", ["arg1"] = "CB_STANDING_NEUTRAL0", },
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."3", faGender) .. " (-3,000 to 0)", ["arg1"] = "CB_STANDING_UNFRIENDLY", },
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."2", faGender) .. " (-6,000 to -3,000)" , ["arg1"] = "CB_STANDING_HOSTILE", },
		{ ["text"] = GetText("FACTION_STANDING_LABEL".."1", faGender) .. " (-42,000 to -6,000)", ["arg1"] = "CB_STANDING_HATED", },
	}

-- Slash commands
SLASH_FACTIONADDICT1 = "/fa"
SLASH_FACTIONADDICT2 = "/factionaddict"

SlashCmdList["FACTIONADDICT"] = function(msg, editbox)
	if frameFactionAddict:IsShown() then
		frameFactionAddict:Hide();
	else
		frameFactionAddict:Show();
	end;
end;

-- example message: Reputation with Lower City increased by 300.
-- globalstrings - FACTION_STANDING_DECREASED - FACTION_STANDING_INCREASED 
local pattern_standing_inc = string.gsub(string.gsub(FACTION_STANDING_INCREASED, "(%%s)", "(.+)"), "(%%d)", "(%%d+)");
local pattern_standing_dec = string.gsub(string.gsub(FACTION_STANDING_DECREASED, "(%%s)", "(.+)"), "(%%d)", "(%%d+)");

--*****************************************************************************
--*****************************************************************************
local function sortedpairs(t) 
  local keys = {} 
  for k in pairs(t) do 
  	table.insert(keys, k) 
  end 
  table.sort(keys, function(a,b) 
      if tonumber(a) and tonumber(b) then 
        return a < b 
      else 
        return tostring(a)<tostring(b) 
      end 
  end) 
  local i = 0 
  return function() 
    if i < #keys then 
      i = i + 1 
      return keys[i], t[keys[i]] 
    end 
  end 
end 

--*****************************************************************************
--*****************************************************************************
local function fa_FindChatWindow(windowname)
	local x;
	
	for x=1,NUM_CHAT_WINDOWS
	do
	  local name, fontSize, r, g, b, alpha, shown, locked, docked, uninteractable = GetChatWindowInfo(x);
	  if (name == windowname) then
		return x;
	  end;
	end;
	-- Default to main chat window if DEBUG window not found
	return 1;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_DebugOut(outString)
	if (debugMode) then
		-- calc delta time
		local prefixStr, deltaTime;
		deltaTime = GetTime() - faStartTime;
		prefixStr = string.format("[%.3f] ", deltaTime);
		-- print to chatframe named "DEBUG" else main chatframe
		getglobal("ChatFrame"..(fa_FindChatWindow("DEBUG"))):AddMessage(prefixStr..outString);
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_ShowFactions()
	local line;
	for line=1,7 do
		getglobal("FactionAddictEntry"..line):Show();
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_HideFactions()
	local line;
	for line=1,7 do
		getglobal("FactionAddictEntry"..line):Hide();
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_ShowLogging()
	local line;
	for line=1,17 do
		getglobal("FactionAddictLogEntry"..line):Show();
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_HideLogging()
	local line;
	for line=1,17 do
		getglobal("FactionAddictLogEntry"..line):Hide();
	end;
end;


--*****************************************************************************
-- Function Credit: Phanx
--*****************************************************************************
local fa_IsFactionInactive, fa_SetWatchedFactionByID, fa_UpdateInactiveFactionsList
do
        function fa_IsFactionInactive(factionID)
                return isInactive[factionID]
        end
        function fa_IsFactionInReputationDisplay(factionID)
        	return isInReputationDisplay[factionID];
        end
        function fa_UpdateInactiveFactionsList()
                if (running) then 
                	return; 
                end
                running = true;
                wipe(isInactive)
                local i = 1
                local inactive
                local wasCollapsed = {}
                while i <= GetNumFactions() do
                        local name, _, _, _, _, _, _, _, isHeader, isCollapsed, _, _, _, factionID = GetFactionInfo(i)
                        if isHeader then
                                if isCollapsed then
                                        ExpandFactionHeader(i)
                                        wasCollapsed[name] = true
                                        if name == FACTION_INACTIVE then
                                        	inactive = true
                                        end
                                elseif name == FACTION_INACTIVE then
                                        inactive = true
                                end
                        elseif inactive then
                                isInactive[factionID] = true
                        end
                        i = i + 1
                        if (factionID ~= nil) then
                        	isInReputationDisplay[factionID] = true;
                        end;
                end
                i = 1
                while i <= GetNumFactions() do
                        local name, _, _, _, _, _, _, _, isHeader, isCollapsed, _, _, _, factionID = GetFactionInfo(i)
                        if isHeader and not isCollapsed and wasCollapsed[name] then
                                CollapseFactionHeader(i)
                                wasCollapsed[name] = nil
                        end
                        i = i + 1
                end
                running = nil
        end
        function fa_SetWatchedFactionByID(watchID)
                if running then return end;
                running = true
                local i = 1
                local wasCollapsed = {}
                while i <= GetNumFactions() do
                        local name, _, _, _, _, _, _, _, isHeader, isCollapsed, _, _, _, factionID = GetFactionInfo(i)
                        if isHeader then
                                if isCollapsed then
                                        ExpandFactionHeader(i)
                                        wasCollapsed[name] = true
                                elseif name == FACTION_INACTIVE then
                                        break
                                end
                        elseif factionID == watchID then
                                SetWatchedFactionIndex(i)
                                break
                        end
                        i = i + 1
                end
                i = 1
                while i <= GetNumFactions() do
                        local name, _, _, _, _, _, _, _, isHeader, isCollapsed, _, _, _, factionID = GetFactionInfo(i)
                        if isHeader and not isCollapsed and wasCollapsed[name] then
                                CollapseFactionHeader(i)
                                wasCollapsed[name] = nil
                        end
                        i = i + 1
                end
                running = nil
        end
end

--*****************************************************************************
--*****************************************************************************
local function fa_BuildTooltipData()

	-- MOP Change - exalted achievements account wide now

	-- 522 - first faction achievement
	local achievementID = 522;
	local i = 1;

	fa_DebugOut("BuildTooltipData");

	while (achievementID ~= nil)
	do
		fa_DebugOut("achievementID: " .. tostring(achievementID));
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch = GetAchievementInfo(achievementID);
		if (id ~= nil) then
			if (tempAchievementTooltipData[i] == nil) then
				tempAchievementTooltipData[i] = {};
			end;
			tempAchievementTooltipData[i][1] = completed;
			fa_DebugOut("completed: " .. tostring(completed));
			tempAchievementTooltipData[i][2] = name;
			i = i + 1;
			local nextID, completed = GetNextAchievement(id);
			achievementID = nextID;
		else
			achievementID = nil;
		end;
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
local function fa_StandingTextByID(standingID)

	local standingTxt;
	local postfixTxt;
	
	standingTxt = GetText("FACTION_STANDING_LABEL"..standingID, UnitSex("player"));

	return(standingTxt);
end;

--*****************************************************************************
--*****************************************************************************
local function fa_GetNextAchievementValue(exaltedCount)
	
	local stringOut;
	
	if (exaltedCount == 0) then
		stringOut = tostring(1);
	elseif (exaltedCount >= 60) then
		stringOut = "NA";
	else -- between 0 and 60
		stringOut = tostring(((math.floor(exaltedCount/5)) + 1) * 5);
	end;
	
	return(stringOut);
end;

--*****************************************************************************
--*****************************************************************************
local function fa_PopulateLoggingTable()
 
	local x,k,v;
	
	-- init vars before rebuilding table
	wipe(tempLoggingData);
	tempLoggingData_rowcount = 0;
	
	-- DEBUG
	fa_DebugOut("fa_PopulateLoggingTable: Start");
	
	-- always populate first line as last reputation gain
	tempLoggingData_rowcount = tempLoggingData_rowcount + 1;
	tempLoggingData[tempLoggingData_rowcount] = {};
	tempLoggingData[tempLoggingData_rowcount][1] = faLastRepGainStr;
	tempLoggingData[tempLoggingData_rowcount][2] = 2;
	
	-- nothing in table - return
	if (FactionAddictLogData == nil or FactionAddictLogEntries == nil) then
		return;
	end;
	
	for x=1,FACTION_ADDICT_LOGGING_DAYS
	do
		-- check for nil date or entry
		if (FactionAddictLogData[x] == nil or FactionAddictLogEntries[x] == nil) then
			return;
		end;
		if (FactionAddictLogData[x]["date"] == nil or FactionAddictLogData[x]["rows"] == nil) then
			return;
		end;
		--check for no rows in the group
		if (FactionAddictLogData[x]["rows"] > 0) then
			-- insert the date if made it this far
			tempLoggingData_rowcount = tempLoggingData_rowcount + 1;
			tempLoggingData[tempLoggingData_rowcount] = {};
			-- row text
			tempLoggingData[tempLoggingData_rowcount][1] = FactionAddictLogData[x]["date"];
			-- row header indicator
			tempLoggingData[tempLoggingData_rowcount][2] = 1;
			fa_DebugOut("PopulateLoggingTable: rowcount: " ..  tostring(tempLoggingData_rowcount) .. " data: " .. tempLoggingData[tempLoggingData_rowcount][1]);
			
			-- Loop through the entries table and build new table
			for k,v in sortedpairs(FactionAddictLogEntries[x]) 
			do 
				tempLoggingData_rowcount = tempLoggingData_rowcount + 1;
				tempLoggingData[tempLoggingData_rowcount] = {};
				-- row text
				if (tempFactionIDsByName[k] ~= nil) then 
					if (v < 0) then
						tempLoggingData[tempLoggingData_rowcount][1] = "[" .. k .. "]" .. ": " .. RED_FONT_COLOR_CODE .. tostring(v) .. FONT_COLOR_CODE_CLOSE;
					else
						tempLoggingData[tempLoggingData_rowcount][1] = "[" .. k .. "]" .. ": " .. tostring(v);
					end;
				else
					fa_DebugOut("PopulateLoggingTable: faction not found - " .. k);
					if (v < 0) then
						tempLoggingData[tempLoggingData_rowcount][1] = k .. ": " .. RED_FONT_COLOR_CODE .. tostring(v) .. FONT_COLOR_CODE_CLOSE;
					else
						tempLoggingData[tempLoggingData_rowcount][1] = k .. ": " .. tostring(v);
					end;
				end;
				-- row header indicator
				tempLoggingData[tempLoggingData_rowcount][2] = 0;
				fa_DebugOut("PopulateLoggingTable: rowcount: " ..  tostring(tempLoggingData_rowcount) .. " data: " .. tempLoggingData[tempLoggingData_rowcount][1]);
			end;
		end;
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_RollLogData()
	local x;
	-- logs stored from 1-FACTION_ADDICT_LOGGING_DAYS; 1 is most current
	for x=FACTION_ADDICT_LOGGING_DAYS,2,-1
	do
		fa_DebugOut("RollLogData: " .. tostring(x));
		if (FactionAddictLogData[x-1] ~= nil) then
			FactionAddictLogData[x] = {};
			FactionAddictLogData[x] = FactionAddictLogData[x-1];
			end;
		if (FactionAddictLogEntries[x-1] ~= nil) then
			FactionAddictLogEntries[x] = {};
			FactionAddictLogEntries[x] = FactionAddictLogEntries[x-1];
		end;
	end;
	-- clear [1]
	FactionAddictLogData[1] = {};
	FactionAddictLogEntries[1] = {};
end;

--*****************************************************************************
--*****************************************************************************
local function fa_AddToRepLog(faction, rep)
	-- faSessionDate - set on addon load
	
	--Check if no data - add new entry for today and new rep
	if (FactionAddictLogData == nil) then
		FactionAddictLogData = {};
	end;
	if (FactionAddictLogData[1] == nil) then
		FactionAddictLogData[1] = {};
	end;
	if (FactionAddictLogData[1]["date"] == nil) then
		FactionAddictLogData[1]["date"] = faSessionDate;
		FactionAddictLogData[1]["rows"] = 0;
	end;
	
	-- If adding a new day roll the log tables
	if (FactionAddictLogData[1]["date"] ~= faSessionDate) then
		fa_RollLogData();
		-- overwrite [1] after rolling
		FactionAddictLogData[1]["date"] = faSessionDate;
		FactionAddictLogData[1]["rows"] = 0;
	end;
	-- add or increment rep for today
	if (FactionAddictLogEntries == nil) then
		FactionAddictLogEntries = {};
	end;
	if (FactionAddictLogEntries[1] == nil) then
		FactionAddictLogEntries[1] = {};
	end;
	if (FactionAddictLogEntries[1][faction] == nil) then
		FactionAddictLogEntries[1][faction] = rep;
		FactionAddictLogData[1]["rows"] = FactionAddictLogData[1]["rows"] + 1;
	else
		FactionAddictLogEntries[1][faction] = FactionAddictLogEntries[1][faction] + rep;
	end;
	fa_PopulateLoggingTable();
	
	if (FactionAddictTab2:IsShown()) then
		FactionAddict_LoggingScrollBar_Update();
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_orderby1(a,b)
	if a[7] == b[7] then
		return a[3] < b[3]
	else 
		return a[7] > b[7]
	end
end
local function fa_orderby2(a,b)
	if a[3] == b[3] then
		return a[7] < b[7]
	else 
		return a[3] < b[3]
	end
end
-- sort by standingId, then barValue, then name
-- with 5.4 - Tillers subFactions cap at 6 Honored
-- workaround from first orderby function
local function fa_orderby3(a,b)
	if a[4] == b[4] then
		if a[7] == b[7] then
			return a[3] < b[3]
		else 
			return a[7] > b[7]
		end
	else 
		return a[4] > b[4]
	end
end

--*****************************************************************************
--*****************************************************************************
local function fa_SortDisplayTable()

	-- name in [3], reputation value in [7]
	if (FactionAddictConfig.RB_ORDERBY == 1) then
		table.sort(tempDisplayData, fa_orderby3);
	else
		table.sort(tempDisplayData, fa_orderby2);
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
local function fa_PopulateAllFactionDataTable()

	local maintableRow;
	

	-- avoid unnecessary rebuilding of table
	if (doServerFactionQuery == 0) then
		return;
	end;
	
	-- init vars before rebuilding table
	tempAllFactionData_rowcount = 0;
	doServerFactionQuery = 0;
	faUpdateDisplayTable = true;
	faNumFactions = 0;
	faNumFactionsExalted = 0;
	faPlayerGuildName = "";
	--wipe(tempAllFactionData);

	-- Update list of inactive factions
	fa_UpdateInactiveFactionsList();

	-- This creates a temp table by joining local faction data and server results
	for maintableRow in ipairs(faFactionData)
	do
		-- Get Faction Info
		local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfoByID(faFactionData[maintableRow][1]);

		if (name ~= nil) then
		
			-- DEBUG -- 
			fa_DebugOut("Faction Addict: Got info for - " .. name .. " id: " ..  tostring(faFactionData[maintableRow][1]) );
		
			-- build a temp table mapping faction names to faction ids
			tempFactionIDsByName[name] = faFactionData[maintableRow][1];
		
			-- calculate percent to exalted
			local percentToExalted = (barValue / 42000)*100;
			-- cap at 100
			if (percentToExalted > 100) then
				percentToExalted = 100;
			end;
			
			if (standingID == 8) then
				faNumFactionsExalted = faNumFactionsExalted + 1;
			end;
			faNumFactions = faNumFactions + 1;
			
			if (tempAllFactionData[name] == nil) then
				tempAllFactionData[name] = {};
			end
			tempAllFactionData_rowcount = tempAllFactionData_rowcount + 1;
			
			-- column 1 - faction id
			tempAllFactionData[name][1] = faFactionData[maintableRow][1];
			-- column 2 - faction icon
			tempAllFactionData[name][2] = faFactionData[maintableRow][2];
			-- column 3 - faction name
			tempAllFactionData[name][3] = name;
			-- column 4 - faction standingID
			tempAllFactionData[name][4] = standingID;
			-- column 5 - faction barMin
			tempAllFactionData[name][5] = barMin;
			-- column 6 - faction barMax
			tempAllFactionData[name][6] = barMax;
			-- column 7 - faction barValue
			tempAllFactionData[name][7] = barValue;
			-- column 8 - faction pctToExalted
			tempAllFactionData[name][8] = string.format("%4.2f%%", percentToExalted);
			-- column 9 - faction description
			tempAllFactionData[name][9] = description;
			-- column 10 - faction expansion
			tempAllFactionData[name][10] = faFactionData[maintableRow][3];
			-- column 11 - faction side
			tempAllFactionData[name][11] = faFactionData[maintableRow][4];
			-- column 12 - faction category
			tempAllFactionData[name][12] = faFactionData[maintableRow][5];
			-- column 13 - hasBonusRepGain
			tempAllFactionData[name][13] = hasBonusRepGain;
			-- column 14 - canBeLFGBonus
			tempAllFactionData[name][14] = canBeLFGBonus;
			
			-- Save players guild name
			if (tempAllFactionData[name][1] == GUILD_FACTION_ID) then
				faPlayerGuildName = tempAllFactionData[name][3];
			end;
		
		end;

	end;
	fa_DebugOut("NumFactions: ".. tostring(faNumFactions));
	fa_DebugOut("NumFactionsExalted: ".. tostring(faNumFactionsExalted));
	-- populate logging table - workaround for guildname issue
	fa_PopulateLoggingTable();
	-- populate achievement tooltip table
	fa_BuildTooltipData();

end;

--*****************************************************************************
--*****************************************************************************
local function fa_PopulateDisplayTable()
	
	local maintableRow;
	local displayThisRow;
	
	fa_DebugOut("fa_PopulateDisplayTable - doServerFactionQuery: " .. tostring(doServerFactionQuery) .. " - faUpdateDisplayTable: " .. tostring(faUpdateDisplayTable));

		-- if the suggest table is nil force an update
	if (tempDisplayData == nil) then
		faUpdateDisplayTable = true;
	end;
	

	-- populate all faction data - serveryquery flag used here
	-- if doServerFactionQuery is true - UpdateDisplayTable flag will be set to true in here
	fa_PopulateAllFactionDataTable();


	-- Only need to populate this table once - and when server data is rechecked or config change
	if (faUpdateDisplayTable == false) then
		return;
	end

	-- init vars before rebuilding table
	wipe(tempDisplayData);
	tempDisplayData_rowcount = 0;
	faUpdateDisplayTable = false;

	faNumFactions = 0;

	-- This creates a temp table by joining local faction data and server results
	for maintableRow in pairs(tempAllFactionData)
	do

		-- Determine if row should be displayed based on config settings
		displayThisRow = false;
		
		local factionID = tempAllFactionData[maintableRow][1];
		local standingID = tempAllFactionData[maintableRow][4];
		local barValue = tempAllFactionData[maintableRow][7];
		local expansion = tempAllFactionData[maintableRow][10];
		local aoho = tempAllFactionData[maintableRow][11];
		
		-- Check standing filter for displayed factions
		if (standingID == 8 and FactionAddictConfig.CB_STANDING_EXALTED == true) then
			displayThisRow = true;
		elseif (standingID == 7 and FactionAddictConfig.CB_STANDING_REVERED == true) then
			displayThisRow = true;
		-- Workaround due to capping of Tillers SubFaction at standing 6
		elseif (standingID == 6) then
			if (barValue >= 42000 and FactionAddictConfig.CB_STANDING_EXALTED == true) then
				displayThisRow = true;
			elseif (barValue >= 21000 and barValue < 42000 and FactionAddictConfig.CB_STANDING_REVERED == true) then
				displayThisRow = true;
			elseif (barValue >= 9000 and barValue < 21000 and FactionAddictConfig.CB_STANDING_HONORED == true) then
				displayThisRow = true;
			elseif (barValue >= 3000 and barValue < 9000 and FactionAddictConfig.CB_STANDING_FRIENDLY == true) then
				displayThisRow = true;
			end;
		elseif (standingID == 5 and FactionAddictConfig.CB_STANDING_FRIENDLY == true) then
			displayThisRow = true;
		elseif (standingID == 4 and barValue > 0 and FactionAddictConfig.CB_STANDING_NEUTRAL1 == true) then
			displayThisRow = true;
		elseif (standingID == 4 and barValue == 0 and FactionAddictConfig.CB_STANDING_NEUTRAL0 == true) then
			displayThisRow = true;
		elseif (standingID == 3 and FactionAddictConfig.CB_STANDING_UNFRIENDLY == true) then
			displayThisRow = true;
		elseif (standingID == 2 and FactionAddictConfig.CB_STANDING_HOSTILE == true) then
			displayThisRow = true;
		elseif (standingID == 1 and FactionAddictConfig.CB_STANDING_HATED == true) then
			displayThisRow = true;
		else
			displayThisRow = false;
		end;
		
		-- Check expansion type filter
		if (displayThisRow) then
			if (expansion == 0 and FactionAddictConfig.CB_CATEGORY_CF == true) then
				displayThisRow = true;
			elseif (expansion == 1 and FactionAddictConfig.CB_CATEGORY_BC == true) then
				displayThisRow = true;
			elseif (expansion == 2 and FactionAddictConfig.CB_CATEGORY_WOTLK == true) then
				displayThisRow = true;
			elseif (expansion == 3 and FactionAddictConfig.CB_CATEGORY_CATA == true) then
				displayThisRow = true;
			elseif (expansion == 4 and FactionAddictConfig.CB_CATEGORY_MOP == true) then
				displayThisRow = true;
			elseif (expansion == 5 and FactionAddictConfig.CB_CATEGORY_WOD == true) then
				displayThisRow = true;
			else
				displayThisRow = false;
			end;
		end;
		
		-- Check alliance/horde filter
		if (displayThisRow) then
			if (aoho == 1 and FactionAddictConfig.CB_FILTER_SHOW_AO == true) then
				displayThisRow = true;
			elseif (aoho == 2 and FactionAddictConfig.CB_FILTER_SHOW_HO == true) then
				displayThisRow = true;
			elseif (aoho == 0) then
				displayThisRow = true;
			else
				displayThisRow = false;
			end;
		end;
		
		-- Check inactive faction
		if (displayThisRow) then
			if (FactionAddictConfig.CB_OPTION_HIDEINACTIVE == true
				and fa_IsFactionInactive(factionID)) then
				displayThisRow = false;
			else
				displayThisRow = true;
			end;
		end;

		if (displayThisRow) then
			-- inc num factions
			faNumFactions = faNumFactions + 1;
			-- inc rowcount for display table
			tempDisplayData_rowcount = tempDisplayData_rowcount + 1;
			-- create a new row
			tempDisplayData[tempDisplayData_rowcount] = {};
			-- copy row from alldata table
			tempDisplayData[tempDisplayData_rowcount] = tempAllFactionData[maintableRow]; 
		end;
		
	end;
	
	-- set text
	fa_DebugOut("NumFactions: ".. tostring(faNumFactions));
	fa_DebugOut("NumFactionsExalted: ".. tostring(faNumFactionsExalted));
	FactionAddictTab1.faFactionsText1:SetText(L.NEXT_ACHIEV_TXT .. HIGHLIGHT_FONT_COLOR_CODE .. fa_GetNextAchievementValue(faNumFactionsExalted) .. FONT_COLOR_CODE_CLOSE);
	FactionAddictTab1.faFactionsText2:SetText(L.SHOWN_TXT .. HIGHLIGHT_FONT_COLOR_CODE .. tostring(faNumFactions) .. FONT_COLOR_CODE_CLOSE .. " - " .. L.EXALTED_TXT .. HIGHLIGHT_FONT_COLOR_CODE .. tostring(faNumFactionsExalted) .. FONT_COLOR_CODE_CLOSE);

	
end;

--*****************************************************************************
--*****************************************************************************
local function fa_SetStatusBarText(offset)

	local outString;
	outString = tostring(tempDisplayData[offset][7]) .. " / " .. tostring(tempDisplayData[offset][6]);
	return outString;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_SetStatusBarText_ByFactionName(faction_name)

	local outString;
	outString = tostring(tempAllFactionData[faction_name][7]) .. " / " .. tostring(tempAllFactionData[faction_name][6]);
	return outString;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_CleanConfigVars()
	
	local key, value;

	-- loop through the client stored variables - remove those no longer used
	for key,value in pairs(FactionAddictConfig) do 
		if (FactionAddictConfigDefaults[key] == nil) then
			FactionAddictConfig[key] = nil;
			fa_DebugOut("Faction Addict - Removing old config value: " .. key);
		end;
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
local function fa_UpdateLogVersion()

	-- FactionAddictLogVersion - version stored in saved variables 
	-- FACTION_ADDICT_LOGGING_VERSION - version for current version of addon

	-- use this function to do any required log data updates
	fa_DebugOut("Faction Addict - Updating Log Version From: " .. FactionAddictLogVersion .. "To: " .. FACTION_ADDICT_LOGGING_VERSION);
	
	-- TODOFUTURE - add update functionality when necessary
	
	-- update stored version
	FactionAddictLogVersion = FACTION_ADDICT_LOGGING_VERSION;
	
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_DisplayTooltip(self)

	local name = self:GetName();
	
	fa_DebugOut("DisplayTooltip: " .. name);	
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	
	-- Show tooltip based on config item name
	GameTooltip:SetText(faFactionAddictTooltips[name], 1.0,1.0,1.0,1,1)
end;

--*****************************************************************************
--*****************************************************************************
local function fa_GetDDConfigValue(param)
	
	if (FactionAddictConfig == nil) then
		return;
	end;
	
	if (param == "CB_STANDING_EXALTED") then
		return(FactionAddictConfig.CB_STANDING_EXALTED);
	elseif (param == "CB_STANDING_REVERED") then
		return(FactionAddictConfig.CB_STANDING_REVERED);
	elseif (param == "CB_STANDING_HONORED") then
		return(FactionAddictConfig.CB_STANDING_HONORED);
	elseif (param == "CB_STANDING_FRIENDLY") then
		return(FactionAddictConfig.CB_STANDING_FRIENDLY);
	elseif (param == "CB_STANDING_NEUTRAL1") then
		return(FactionAddictConfig.CB_STANDING_NEUTRAL1);
	elseif (param == "CB_STANDING_NEUTRAL0") then
		return(FactionAddictConfig.CB_STANDING_NEUTRAL0);
	elseif (param == "CB_STANDING_UNFRIENDLY") then
		return(FactionAddictConfig.CB_STANDING_UNFRIENDLY);
	elseif (param == "CB_STANDING_HOSTILE") then
		return(FactionAddictConfig.CB_STANDING_HOSTILE);
	elseif (param == "CB_STANDING_HATED") then
		return(FactionAddictConfig.CB_STANDING_HATED);
	else
		return(false);
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_SetDDConfigValue(self, arg1, arg2, checked)
	PlaySound("igInventoryRotateCharacter");
	
	if (arg1 == "CB_STANDING_EXALTED") then
		FactionAddictConfig.CB_STANDING_EXALTED = checked;
		faUpdateDisplayTable = true;
	elseif (arg1 == "CB_STANDING_REVERED") then
		FactionAddictConfig.CB_STANDING_REVERED = checked;
		faUpdateDisplayTable = true;
	elseif (arg1 == "CB_STANDING_HONORED") then
		FactionAddictConfig.CB_STANDING_HONORED = checked;
		faUpdateDisplayTable = true;
	elseif (arg1 == "CB_STANDING_FRIENDLY") then
		FactionAddictConfig.CB_STANDING_FRIENDLY = checked;
		faUpdateDisplayTable = true;
	elseif (arg1 == "CB_STANDING_NEUTRAL1") then
		FactionAddictConfig.CB_STANDING_NEUTRAL1 = checked;
		faUpdateDisplayTable = true;
	elseif (arg1 == "CB_STANDING_NEUTRAL0") then
		FactionAddictConfig.CB_STANDING_NEUTRAL0 = checked;
		faUpdateDisplayTable = true;
	elseif (arg1 == "CB_STANDING_UNFRIENDLY") then
		FactionAddictConfig.CB_STANDING_UNFRIENDLY = checked;
		faUpdateDisplayTable = true;
	elseif (arg1 == "CB_STANDING_HOSTILE") then
		FactionAddictConfig.CB_STANDING_HOSTILE = checked;
		faUpdateDisplayTable = true;
	elseif (arg1 == "CB_STANDING_HATED") then
		FactionAddictConfig.CB_STANDING_HATED = checked;
		faUpdateDisplayTable = true;
	else
		return(false);
	end;
	
	-- after updating config refresh display
	-- populate the display table
	fa_PopulateDisplayTable();
	-- order the display
	fa_SortDisplayTable();
	-- display the data
	if (FactionAddictTab1:IsShown()) then
		FactionAddict_ScrollBar_Update();
	end;
	
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_AchText_OnEnter(self)

	local tableRow;

	-- Populate Achievement Completion tooltip
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 15, -95);
	GameTooltip:SetText(YELLOW_FONT_COLOR_CODE..L.ACHIEVEMENT_TT_TITLE..FONT_COLOR_CODE_CLOSE);
	GameTooltip:AddLine(" ");
	
	-- NOTE - there is currently a limit of 10 textures in the GameToolTip
	-- since this needs 11 show none at this time

	for tableRow in ipairs(tempAchievementTooltipData)
	do
		-- Check if achievement completed
		if (tempAchievementTooltipData[tableRow][1] == true) then
			GameTooltip:AddLine(tempAchievementTooltipData[tableRow][2], GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b, 1);
			--GameTooltip:AddTexture("Interface\\RAIDFRAME\\ReadyCheck-Ready");
		else
			GameTooltip:AddLine(tempAchievementTooltipData[tableRow][2], GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b, 1);
			--GameTooltip:AddTexture(" ");
		end;
	end;
	
	GameTooltip:Show();
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_ScrollBar_Update()
	local line; -- 1 through 7 of our window to scroll
	local lineplusoffset; -- an index into our data calculated from the scroll offset
	local color; -- status bar color

	fa_DebugOut("FactionAddict_ScrollBar_Update() - display rowcount: " .. tempDisplayData_rowcount);

	FauxScrollFrame_Update(FactionAddictScrollBar,tempDisplayData_rowcount,7,44);
	for line=1,7 
	do
		lineplusoffset = line + FauxScrollFrame_GetOffset(FactionAddictScrollBar);
		
		-- set alpha values for each row
		if (tempDisplayData_rowcount == 0) then
			getglobal("FactionAddictEntry"..line).FactionAddictButtonBack:Hide();
			getglobal("FactionAddictEntry"..line).FactionAddictButtonHighlight:Hide();
		else
			getglobal("FactionAddictEntry"..line).FactionAddictButtonBack:Show();
			getglobal("FactionAddictEntry"..line).FactionAddictButtonHighlight:Show();
			getglobal("FactionAddictEntry"..line).FactionAddictButtonBack:SetAlpha(0.35);
			getglobal("FactionAddictEntry"..line).FactionAddictButtonHighlight:SetAlpha(0.70);
		end;
		
		if lineplusoffset <= tempDisplayData_rowcount then
			getglobal("FactionAddictEntry"..line):Enable();
			getglobal("FactionAddictEntry"..line).FactionNameTxt:SetText(tempDisplayData[lineplusoffset][3]);
			getglobal("FactionAddictEntry"..line).FactionNameTxt:Show();
			
			-- set faction icon texture
			getglobal("FactionAddictEntry"..line).FactionIcon:SetTexture(tempDisplayData[lineplusoffset][2]);
			getglobal("FactionAddictEntry"..line).FactionIcon:Show();
			
			-- set faction icon overlay texture
			if (tempDisplayData[lineplusoffset][4] == 8) then
				getglobal("FactionAddictEntry"..line).FactionIconOverlay:Show();
			else
				getglobal("FactionAddictEntry"..line).FactionIconOverlay:Hide();
			end;
			
			-- set faction side icon
			if (tempDisplayData[lineplusoffset][11] == 2) then
				getglobal("FactionAddictEntry"..line).SideFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Horde.png");
				getglobal("FactionAddictEntry"..line).SideFactionIcon:Show();
				getglobal("FactionAddictEntry"..line).SideFactionIcon:SetDrawLayer("OVERLAY",7);
			elseif (tempDisplayData[lineplusoffset][11] == 1) then
				getglobal("FactionAddictEntry"..line).SideFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Alliance.png");
				getglobal("FactionAddictEntry"..line).SideFactionIcon:Show();
				getglobal("FactionAddictEntry"..line).SideFactionIcon:SetDrawLayer("OVERLAY",7);
			else
				getglobal("FactionAddictEntry"..line).SideFactionIcon:Hide();
			end;
			
			
			-- set percent
			getglobal("FactionAddictEntry"..line).FactionPctTxt:SetText(tempDisplayData[lineplusoffset][8]);
			getglobal("FactionAddictEntry"..line).FactionPctTxt:Show();
			
			-- set progress bar
			getglobal("FactionAddictEntry"..line).FAReputationBar:SetMinMaxValues(tempDisplayData[lineplusoffset][5], tempDisplayData[lineplusoffset][6]);
			getglobal("FactionAddictEntry"..line).FAReputationBar:SetValue(tempDisplayData[lineplusoffset][7]);
			getglobal("FactionAddictEntry"..line).FAReputationBar:Show();
			
			-- set color
			color = FACTION_BAR_COLORS[tempDisplayData[lineplusoffset][4]];
			getglobal("FactionAddictEntry"..line).FAReputationBar:SetStatusBarColor(color.r, color.g, color.b);
			-- set progress bar text
			getglobal("FactionAddictEntry"..line).FAReputationBar.text:SetText(fa_StandingTextByID(tempDisplayData[lineplusoffset][4]));
			getglobal("FactionAddictEntry"..line).FAReputationBar.text:Show();
			
			getglobal("FactionAddictEntry"..line).FAReputationBar.text2:SetText(fa_SetStatusBarText(lineplusoffset));
			getglobal("FactionAddictEntry"..line).FAReputationBar.text2:Hide();
			
			-- set bonus icon
			if (tempDisplayData[lineplusoffset][13] == true) then
				getglobal("FactionAddictEntry"..line).FAReputationBar.FABonusIcon:Show();
			else
				getglobal("FactionAddictEntry"..line).FAReputationBar.FABonusIcon:Hide();
			end;

			-- set LFG Bonus Rep button
			if (fa_IsFactionInReputationDisplay(tempDisplayData[lineplusoffset][1])) then 
				getglobal("FactionAddictEntry"..line).FALFGBonusRepButton.factionID = tempDisplayData[lineplusoffset][1];
				getglobal("FactionAddictEntry"..line).FALFGBonusRepButton:SetShown(tempDisplayData[lineplusoffset][14]);
	 			getglobal("FactionAddictEntry"..line).FALFGBonusRepButton:SetChecked(lfgBonusFactionID == tempDisplayData[lineplusoffset][1]);
	 			getglobal("FactionAddictEntry"..line).FALFGBonusRepButton:SetEnabled(lfgBonusFactionID ~= tempDisplayData[lineplusoffset][1]);
	 			getglobal("FactionAddictEntry"..line).FALFGBonusRepButton.Glow:Show();
	 			getglobal("FactionAddictEntry"..line).FALFGBonusRepButton.Glow:SetAlpha(0.18);
	 			getglobal("FactionAddictEntry"..line).FALFGBonusRepButton.GlowAnim:Stop();
	 		else
	 			getglobal("FactionAddictEntry"..line).FALFGBonusRepButton:Disable();
				getglobal("FactionAddictEntry"..line).FALFGBonusRepButton:Hide();
 			end

			-- set expansion icon (ExpansionIcon)
			if (tempDisplayData[lineplusoffset][10] == 1) then -- BC
				getglobal("FactionAddictEntry"..line).ExpansionIcon:SetTexCoord(0.0, 0.546875, 0.0, 0.1171875);
				getglobal("FactionAddictEntry"..line).ExpansionIcon:Show();
			elseif (tempDisplayData[lineplusoffset][10] == 2) then -- WOTLK
				getglobal("FactionAddictEntry"..line).ExpansionIcon:SetTexCoord(0.0, 0.546875, 0.1171875, 0.234375);
				getglobal("FactionAddictEntry"..line).ExpansionIcon:Show();
			elseif (tempDisplayData[lineplusoffset][10] == 3) then -- Cata
				getglobal("FactionAddictEntry"..line).ExpansionIcon:SetTexCoord(0.0, 0.546875, 0.234375, 0.3515625);
				getglobal("FactionAddictEntry"..line).ExpansionIcon:Show();
			elseif (tempDisplayData[lineplusoffset][10] == 4) then -- MOP
				getglobal("FactionAddictEntry"..line).ExpansionIcon:SetTexCoord(0.0, 0.546875, 0.3515625, 0.46875);
				getglobal("FactionAddictEntry"..line).ExpansionIcon:Show();
			elseif (tempDisplayData[lineplusoffset][10] == 5) then -- WOD 
				getglobal("FactionAddictEntry"..line).ExpansionIcon:SetTexCoord(0.0, 0.546875, 0.46875, 0.5859375);
				getglobal("FactionAddictEntry"..line).ExpansionIcon:Show();
			else -- classic
				getglobal("FactionAddictEntry"..line).ExpansionIcon:Hide();
			end;
		else
			getglobal("FactionAddictEntry"..line):Disable();
			getglobal("FactionAddictEntry"..line).FactionNameTxt:Hide();
			getglobal("FactionAddictEntry"..line).FactionIcon:Hide();
			getglobal("FactionAddictEntry"..line).FactionIconOverlay:Hide();
			getglobal("FactionAddictEntry"..line).FactionPctTxt:Hide();
			getglobal("FactionAddictEntry"..line).FAReputationBar:Hide();
			getglobal("FactionAddictEntry"..line).FAReputationBar.FABonusIcon:Hide();
			getglobal("FactionAddictEntry"..line).SideFactionIcon:Hide();
			getglobal("FactionAddictEntry"..line).ExpansionIcon:Hide();
			getglobal("FactionAddictEntry"..line).FALFGBonusRepButton:Disable();
			getglobal("FactionAddictEntry"..line).FALFGBonusRepButton:Hide();
		end;
	end;
	
	-- inform user nothing to show
	if (tempDisplayData_rowcount == 0) then
		-- Show error message
		FactionAddictTab1.faErrorText1:Show();
		FactionAddictTab1.faErrorText1:SetText(L.ERROR1);
		-- Hide scroll bar -- 
		FactionAddictTab1FactionAddictScrollBar:Hide();
	else
		FactionAddictTab1.faErrorText1:Hide();
		FactionAddictTab1FactionAddictScrollBar:Show();
	end;
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_LogEntry_OnClick(self)
	-- returns the log row that was clicked 1-17
	local clickedEntry = tonumber(strsub(self:GetName(),strlen("FactionAddictLogEntry")+1));
	local tablerow;
	fa_DebugOut("Clicked log row: " .. tostring(clickedEntry));
	-- tempLoggingData
	tablerow = clickedEntry + FauxScrollFrame_GetOffset(FactionAddictLoggingScrollBar);
	if (tablerow <= tempLoggingData_rowcount) then
		local logText = tempLoggingData[tablerow][1];
		local s1, e1 = string.find(logText, "%[");
		local s2, e2 = string.find(logText, "%]");
		-- Make sure faction name is found in string
		if (e1 ~= nil and s2 ~= nil) then
			local factionText = string.sub(logText, e1+1, s2-1);
			fa_DebugOut("faction name: " .. factionText);
			faClickedFactionName = factionText;
			PlaySound("igInventoryRotateCharacter");
			frameFactionAddictInfoFrame:Hide();
			frameFactionAddictInfoFrame:Show();
		end;
	end;
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_LoggingScrollBar_Update()
	local line; -- 1 through 17 of our window to scroll
	local lineplusoffset; -- an index into our data calculated from the scroll offset
	
	FauxScrollFrame_Update(FactionAddictLoggingScrollBar,tempLoggingData_rowcount,17,16);
	
	for line=1,17
	do
		lineplusoffset = line + FauxScrollFrame_GetOffset(FactionAddictLoggingScrollBar);
		
		--fa_DebugOut("Updating Logging - line/lineplusoffset: " .. tostring(line) .. "/" .. tostring(lineplusoffset));
		
		if lineplusoffset <= tempLoggingData_rowcount then
			-- check if row is a header
			if (tempLoggingData[lineplusoffset][2] == 1) then
				getglobal("FactionAddictLogEntry"..line).LogEntryIcon:Hide();
				getglobal("FactionAddictLogEntry"..line).LogEntryHdr:Show();
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:Show();
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:SetText(tempLoggingData[lineplusoffset][1]);
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			-- check if row is message
			elseif (tempLoggingData[lineplusoffset][2] == 2) then
				getglobal("FactionAddictLogEntry"..line).LogEntryIcon:Show();
				getglobal("FactionAddictLogEntry"..line).LogEntryHdr:Hide();
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:Show();
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:SetText(tempLoggingData[lineplusoffset][1]);
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			-- else row is info
			else
				getglobal("FactionAddictLogEntry"..line).LogEntryIcon:Hide();
				getglobal("FactionAddictLogEntry"..line).LogEntryHdr:Hide();
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:Show();
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:SetText("  - " .. tempLoggingData[lineplusoffset][1]);
				getglobal("FactionAddictLogEntry"..line).LogEntryTxt:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			end;
		-- else blank
		else
			getglobal("FactionAddictLogEntry"..line).LogEntryHdr:Hide();
			getglobal("FactionAddictLogEntry"..line).LogEntryIcon:Hide();
			getglobal("FactionAddictLogEntry"..line).LogEntryTxt:Hide();
		end;
	end;
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_SetConfigVariables(self)
	
	local configName = self:GetName();
	
	PlaySound("igInventoryRotateCharacter");

	if (configName == "CheckButton_CF") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_CATEGORY_CF = true;
		else
			FactionAddictConfig.CB_CATEGORY_CF = false;
		end;
	elseif (configName == "CheckButton_BC") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_CATEGORY_BC = true;
		else
			FactionAddictConfig.CB_CATEGORY_BC = false;
		end;
	elseif (configName == "CheckButton_WOTLK") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_CATEGORY_WOTLK = true;
		else
			FactionAddictConfig.CB_CATEGORY_WOTLK = false;
		end;
	elseif (configName == "CheckButton_CATA") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_CATEGORY_CATA = true;
		else
			FactionAddictConfig.CB_CATEGORY_CATA = false;
		end;
	elseif (configName == "CheckButton_MOP") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_CATEGORY_MOP = true;
		else
			FactionAddictConfig.CB_CATEGORY_MOP = false;
		end;
	elseif (configName == "CheckButton_WOD") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_CATEGORY_WOD = true;
		else
			FactionAddictConfig.CB_CATEGORY_WOD = false;
		end;
	elseif (configName == "CheckButton_OrderPct") then
		self:SetChecked(true);
		CheckButton_OrderName:SetChecked(false);
		FactionAddictConfig.RB_ORDERBY = 1;
	elseif (configName == "CheckButton_OrderName") then
		self:SetChecked(true);
		CheckButton_OrderPct:SetChecked(false);
		FactionAddictConfig.RB_ORDERBY = 2;
	elseif (configName == "CheckButton_AO") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_FILTER_SHOW_AO = true;
		else
			FactionAddictConfig.CB_FILTER_SHOW_AO = false;
		end;
	elseif (configName == "CheckButton_HO") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_FILTER_SHOW_HO = true;
		else
			FactionAddictConfig.CB_FILTER_SHOW_HO = false;
		end;
	elseif (configName == "CheckButton_AUTOBAR") then
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_OPTION_AUTOBAR = true;
		else
			FactionAddictConfig.CB_OPTION_AUTOBAR = false;
		end;
	elseif (configName == "CheckButton_HIDEINACTIVE") then
		faUpdateDisplayTable = true;
		if (self:GetChecked() == true) then
			FactionAddictConfig.CB_OPTION_HIDEINACTIVE = true;
		else
			FactionAddictConfig.CB_OPTION_HIDEINACTIVE = false;
		end;
	end;
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_FactionEntrySB_OnClick(self)
	
	local clickedEntry = tonumber(strsub(self:GetParent():GetName(),-1));
	local faClickedRow;
	
	fa_DebugOut ("FactionEntry - OnClick: " .. tostring(clickedEntry));
	
	faClickedRow = clickedEntry + FauxScrollFrame_GetOffset(FactionAddictScrollBar);
	
	faClickedFactionName =  tempDisplayData[faClickedRow][3];
	
	PlaySound("igInventoryRotateCharacter");
	frameFactionAddictInfoFrame:Hide();
	frameFactionAddictInfoFrame:Show();

	
end;
--*****************************************************************************
function FactionAddict_FactionEntry_OnClick(self)
	
	local clickedEntry = tonumber(strsub(self:GetName(),-1));
	local faClickedRow;
	
	fa_DebugOut ("FactionEntry - OnClick: " .. tostring(clickedEntry));
	
	faClickedRow = clickedEntry + FauxScrollFrame_GetOffset(FactionAddictScrollBar);
	
	faClickedFactionName =  tempDisplayData[faClickedRow][3];
	
	PlaySound("igInventoryRotateCharacter");
	frameFactionAddictInfoFrame:Hide();
	frameFactionAddictInfoFrame:Show();
	
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_Tab_OnClick(self)
	
	local tabNumber;
	tabNumber = (self:GetID());
	
	PanelTemplates_SetTab(frameFactionAddict, tabNumber);
	
	-- DEBUG
	fa_DebugOut("FactionAddict_Tab_OnClick: " .. tostring(tabNumber));
	
	if ( tabNumber == 1 ) then
		-- Factions tab clicked
		FactionAddictTab1:Show();
		FactionAddictTab2:Hide();
		FactionAddictTab3:Hide();
		FactionAddictTab4:Hide();
		FactionAddictScrollBar:Show();
		FactionAddictLoggingScrollBar:Hide();
		fa_ShowFactions();
		fa_HideLogging();
		-- populate the display table
		fa_PopulateDisplayTable();
		-- order the display
		fa_SortDisplayTable();
		FactionAddict_ScrollBar_Update();
	elseif ( tabNumber == 2 ) then
		-- Log tab clicked
		FactionAddictTab1:Hide();
		FactionAddictTab2:Show();
		FactionAddictTab3:Hide();
		FactionAddictTab4:Hide();
		FactionAddictScrollBar:Hide();
		FactionAddictLoggingScrollBar:Show();
		fa_HideFactions();
		fa_ShowLogging();
	elseif ( tabNumber == 3 ) then
		-- Config tab clicked
		FactionAddictTab1:Hide();
		FactionAddictTab2:Hide();
		FactionAddictTab3:Show();
		FactionAddictTab4:Hide();
		FactionAddictScrollBar:Hide();
		FactionAddictLoggingScrollBar:Hide();
		fa_HideFactions();
		fa_HideLogging();
		-- set checkbox values
		CheckButton_CF:SetChecked(FactionAddictConfig.CB_CATEGORY_CF);
		CheckButton_BC:SetChecked(FactionAddictConfig.CB_CATEGORY_BC);
		CheckButton_WOTLK:SetChecked(FactionAddictConfig.CB_CATEGORY_WOTLK);
		CheckButton_CATA:SetChecked(FactionAddictConfig.CB_CATEGORY_CATA);
		CheckButton_MOP:SetChecked(FactionAddictConfig.CB_CATEGORY_MOP);
		CheckButton_WOD:SetChecked(FactionAddictConfig.CB_CATEGORY_WOD);
		CheckButton_AO:SetChecked(FactionAddictConfig.CB_FILTER_SHOW_AO);
		CheckButton_HO:SetChecked(FactionAddictConfig.CB_FILTER_SHOW_HO);
		CheckButton_AUTOBAR:SetChecked(FactionAddictConfig.CB_OPTION_AUTOBAR);
		CheckButton_HIDEINACTIVE:SetChecked(FactionAddictConfig.CB_OPTION_HIDEINACTIVE);
		if (FactionAddictConfig.RB_ORDERBY == 1) then
			CheckButton_OrderPct:SetChecked(true);
			CheckButton_OrderName:SetChecked(false);
		else
			CheckButton_OrderPct:SetChecked(false);
			CheckButton_OrderName:SetChecked(true);
		end;
	elseif ( tabNumber == 4 ) then
		-- About tab clicked
		FactionAddictTab1:Hide();
		FactionAddictTab2:Hide();
		FactionAddictTab3:Hide();
		FactionAddictTab4:Show();
		FactionAddictScrollBar:Hide();
		FactionAddictLoggingScrollBar:Hide();
		fa_HideFactions();
		fa_HideLogging();
	end;
	PlaySound("igCharacterInfoTab");
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_StatusBar_OnEnter(self)
	--fa_DebugOut("StatusBar_OnEnter - ".. self:GetParent():GetName());
	self.text:Hide();
	self.text2:Show();
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_StatusBar_OnLeave(self)
	--fa_DebugOut("StatusBar_OnLeave - ".. self:GetParent():GetName());
	self.text:Show();
	self.text2:Hide();
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddictLauncher_OnClick(self)

	PlaySound("igMainMenuOptionCheckBoxOn");
	
	if frameFactionAddict:IsShown() then
		frameFactionAddict:Hide()
	else
		frameFactionAddict:Show()
	end;
end;

--*****************************************************************************
--*****************************************************************************
local function fa_InfoWindow_LoadInfo(self)
	
	-- tempAllFactionData has all data
	-- tempDisplayData has data shown in faction tabdisplay

	-- Set icon texture
	self.FactionIconInfoWin:SetTexture(tempAllFactionData[faClickedFactionName][2]);
	
	-- Set Faction Name
	self.FactionNameTxt:SetText(tempAllFactionData[faClickedFactionName][3]);
	-- Set Status Bar values
	local color = FACTION_BAR_COLORS[tempAllFactionData[faClickedFactionName][4]];
	self.StatusBar:SetStatusBarColor(color.r, color.g, color.b, 1);
	self.StatusBar:SetMinMaxValues(tempAllFactionData[faClickedFactionName][5], tempAllFactionData[faClickedFactionName][6]);
	self.StatusBar:SetValue(tempAllFactionData[faClickedFactionName][7]);
	-- Set Status Bar text
	self.StatusBar.text:SetText(fa_StandingTextByID(tempAllFactionData[faClickedFactionName][4]));
	self.StatusBar.text:Show();
	self.StatusBar.text2:SetText(fa_SetStatusBarText_ByFactionName(faClickedFactionName));
	self.StatusBar.text2:Hide();
	-- Set EditBox text
	self.EditBox:SetText(L.FACTION_URL_TXT .. tostring(tempAllFactionData[faClickedFactionName][1]));
	-- Set Percent text
	self.PctTxt:SetText(LIGHTYELLOW_FONT_COLOR_CODE .. L.PCT_TO_EXALTED_TXT .. FONT_COLOR_CODE_CLOSE .. tempAllFactionData[faClickedFactionName][8]);
	-- Set Category text
	self.CategoryTxt:SetText(LIGHTYELLOW_FONT_COLOR_CODE .. L.CATEGORY_TXT .. FONT_COLOR_CODE_CLOSE .. faFactionDataCategoryValues[tempAllFactionData[faClickedFactionName][12]]);
	-- Set Desc text
	self.DescTxt:SetText(tempAllFactionData[faClickedFactionName][9]);
	-- set faction side icon
	if (tempAllFactionData[faClickedFactionName][11] == 2) then
		self.SideFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Horde.png");
		self.SideFactionIcon:Show();
		self.SideFactionIcon:SetDrawLayer("OVERLAY",7);
	elseif (tempAllFactionData[faClickedFactionName][11] == 1) then
		self.SideFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Alliance.png");
		self.SideFactionIcon:Show();
		self.SideFactionIcon:SetDrawLayer("OVERLAY",7);
	else
		self.SideFactionIcon:Hide();
	end;
	-- set bonus icon
	if (tempAllFactionData[faClickedFactionName][13] == true) then
		self.StatusBar.FABonusIcon:Show();
	else
		self.StatusBar.FABonusIcon:Hide();
	end;
	-- display/hide of LFG Bonus Star
	if (fa_IsFactionInReputationDisplay(tempAllFactionData[faClickedFactionName][1])) then 
		self.FALFGBonusRepButton.factionID = tempAllFactionData[faClickedFactionName][1];
		self.FALFGBonusRepButton:SetShown(tempAllFactionData[faClickedFactionName][14]);
		self.FALFGBonusRepButton:SetChecked(lfgBonusFactionID == tempAllFactionData[faClickedFactionName][1]);
		self.FALFGBonusRepButton:SetEnabled(lfgBonusFactionID ~= tempAllFactionData[faClickedFactionName][1]);
		self.FALFGBonusRepButton.Glow:Show();
		self.FALFGBonusRepButton.Glow:SetAlpha(0.18);
		self.FALFGBonusRepButton.GlowAnim:Stop();
	else
		self.FALFGBonusRepButton:Disable();
		self.FALFGBonusRepButton:Hide();
	end
end;

--*****************************************************************************
--*****************************************************************************
local function fa_InfoWindow_Update()

	-- should only be called if window is visible -- after refreshing faction data
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_InfoWindow_OnShow(self)

	fa_DebugOut("FactionAddict_InfoWindow_OnShow: "..self:GetName());
	
	fa_InfoWindow_LoadInfo(self);
	FactionAddictLaunchWatchPanel:SetText(L.LAUNCH_WATCH_PANEL_TXT);
	
end;

--*****************************************************************************
--*****************************************************************************

function FactionAddict_InfoWindow_Btn1_OnClick(self)
	PlaySound("igMainMenuOption");
	frameFactionAddictWatchFrame:Show();
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_OnShow(self)

	PlaySound("igCharacterInfoOpen");
	fa_DebugOut("FactionAddict_OnShow: "..self:GetName());
	
	-- populate the display table
	fa_PopulateDisplayTable();
	-- order the display
	fa_SortDisplayTable();
	-- display the data
	if (FactionAddictTab1:IsShown()) then
		FactionAddict_ScrollBar_Update();
	end;
	-- update info window if displayed
	if (frameFactionAddictInfoFrame:IsVisible()) then
		fa_InfoWindow_LoadInfo(frameFactionAddictInfoFrame);
	end;
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_OnHide(self)

	PlaySound("igCharacterInfoClose");
	fa_DebugOut("FactionAddict_OnHide: "..self:GetName());
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_OnEvent(self, event, ...)
	
	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11 = ...;

	-- Things to do after addon is loaded
	if (event == "ADDON_LOADED") then
		if (arg1 == "FactionAddict") then
			fa_DebugOut("FactionAddict_OnEvent: ".. arg1 .. " " .. event);
			
			-- Check the version of stored logging info
			if (FactionAddictLogVersion == nil) then
				FactionAddictLogVersion = FACTION_ADDICT_LOGGING_VERSION;
			end;
			if (FactionAddictLogVersion < FACTION_ADDICT_LOGGING_VERSION) then
				-- update/clear logging info if necessary
				fa_UpdateLogVersion();
			end;
			
			-- check for nil global config values - if not exist then default to on (1)
			if (FactionAddictConfig == nil) then
				FactionAddictConfig = FactionAddictConfigDefaults;
			end;
			-- check for nil on each config item - in case a new one is added
			if (FactionAddictConfig.CB_STANDING_EXALTED == nil) then
				FactionAddictConfig.CB_STANDING_EXALTED = FactionAddictConfigDefaults.CB_STANDING_EXALTED;
			end;
			if (FactionAddictConfig.CB_STANDING_REVERED == nil) then
				FactionAddictConfig.CB_STANDING_REVERED = FactionAddictConfigDefaults.CB_STANDING_REVERED;
			end;
			if (FactionAddictConfig.CB_STANDING_HONORED == nil) then
				FactionAddictConfig.CB_STANDING_HONORED = FactionAddictConfigDefaults.CB_STANDING_HONORED;
			end;
			if (FactionAddictConfig.CB_STANDING_FRIENDLY == nil) then
				FactionAddictConfig.CB_STANDING_FRIENDLY = FactionAddictConfigDefaults.CB_STANDING_FRIENDLY;
			end;
			if (FactionAddictConfig.CB_STANDING_NEUTRAL1 == nil) then
				FactionAddictConfig.CB_STANDING_NEUTRAL1 = FactionAddictConfigDefaults.CB_STANDING_NEUTRAL1;
			end;
			if (FactionAddictConfig.CB_STANDING_NEUTRAL0 == nil) then
				FactionAddictConfig.CB_STANDING_NEUTRAL0 = FactionAddictConfigDefaults.CB_STANDING_NEUTRAL0;
			end;
			if (FactionAddictConfig.CB_STANDING_UNFRIENDLY == nil) then
				FactionAddictConfig.CB_STANDING_UNFRIENDLY = FactionAddictConfigDefaults.CB_STANDING_UNFRIENDLY;
			end;
			if (FactionAddictConfig.CB_STANDING_HOSTILE == nil) then
				FactionAddictConfig.CB_STANDING_HOSTILE = FactionAddictConfigDefaults.CB_STANDING_HOSTILE;
			end;
			if (FactionAddictConfig.CB_STANDING_HATED == nil) then
				FactionAddictConfig.CB_STANDING_HATED = FactionAddictConfigDefaults.CB_STANDING_HATED;
			end;
			if (FactionAddictConfig.CB_CATEGORY_CF == nil) then
				FactionAddictConfig.CB_CATEGORY_CF = FactionAddictConfigDefaults.CB_CATEGORY_CF;
			end;
			if (FactionAddictConfig.CB_CATEGORY_BC == nil) then
				FactionAddictConfig.CB_CATEGORY_BC = FactionAddictConfigDefaults.CB_CATEGORY_BC;
			end;
			if (FactionAddictConfig.CB_CATEGORY_WOTLK == nil) then
				FactionAddictConfig.CB_CATEGORY_WOTLK = FactionAddictConfigDefaults.CB_CATEGORY_WOTLK;
			end;
			if (FactionAddictConfig.CB_CATEGORY_CATA == nil) then
				FactionAddictConfig.CB_CATEGORY_CATA = FactionAddictConfigDefaults.CB_CATEGORY_CATA;
			end;
			if (FactionAddictConfig.CB_CATEGORY_MOP == nil) then
				FactionAddictConfig.CB_CATEGORY_MOP = FactionAddictConfigDefaults.CB_CATEGORY_MOP;
			end;
			if (FactionAddictConfig.CB_CATEGORY_WOD == nil) then
				FactionAddictConfig.CB_CATEGORY_WOD = FactionAddictConfigDefaults.CB_CATEGORY_WOD;
			end;
			if (FactionAddictConfig.RB_ORDERBY == nil) then
				FactionAddictConfig.RB_ORDERBY = FactionAddictConfigDefaults.RB_ORDERBY;
			end;
			if (FactionAddictConfig.CB_FILTER_SHOW_AO == nil) then
				FactionAddictConfig.CB_FILTER_SHOW_AO = FactionAddictConfigDefaults.CB_FILTER_SHOW_AO;
			end;
			if (FactionAddictConfig.CB_FILTER_SHOW_HO == nil) then
				FactionAddictConfig.CB_FILTER_SHOW_HO = FactionAddictConfigDefaults.CB_FILTER_SHOW_HO;
			end;
			if (FactionAddictConfig.CB_OPTION_AUTOBAR == nil) then
				FactionAddictConfig.CB_OPTION_AUTOBAR = FactionAddictConfigDefaults.CB_OPTION_AUTOBAR;
			end;
			if (FactionAddictConfig.CB_OPTION_HIDEINACTIVE == nil) then
				FactionAddictConfig.CB_OPTION_HIDEINACTIVE = FactionAddictConfigDefaults.CB_OPTION_HIDEINACTIVE;
			end;

			-- cleanup old config vars
			fa_CleanConfigVars();
			
		end;
	end;
	
	if ( (event == "UPDATE_FACTION") ) then

		-- this is a bit of a workaround since value is not set when ADDON_LOADED is fired
		lfgBonusFactionID = GetLFGBonusFactionID();
		if (lfgBonusFactionID ~= nil) then
			fa_DebugOut("UPDATE_FACTION:lfgBonusFactionID: ".. lfgBonusFactionID);
		else
			fa_DebugOut("UPDATE_FACTION:lfgBonusFactionID: nil");
		end

		-- collapsing and expanding default UI also triggers this event
		fa_DebugOut("FactionAddict_OnEvent: " .. event);
		if (arg1 ~= nil) then fa_DebugOut("arg1: " .. arg1); end;

		if (running) then 
			fa_DebugOut("IGNORE UPDATE_FACTION - Already Running!");
			return;
		end;

		-- check if main window is displayed
		if (frameFactionAddict:IsVisible()) then
			-- update data for factions
			doServerFactionQuery = 1;
			fa_PopulateDisplayTable();

			-- order the display
			fa_SortDisplayTable();
			-- display the data
			if (FactionAddictTab1:IsShown()) then
				FactionAddict_ScrollBar_Update();
			end;
			-- update info window if displayed
			if (frameFactionAddictInfoFrame:IsVisible()) then
				fa_InfoWindow_LoadInfo(frameFactionAddictInfoFrame);
			end;
		else -- not visible - just update flag to reload data
			doServerFactionQuery = 1;
		end;

	end;

	if (event == "PLAYER_LOGIN") then
		fa_DebugOut("FactionAddict_OnEvent: " .. event);
		-- check if main window is displayed
		if (frameFactionAddict:IsVisible()) then
			-- update data for factions
			doServerFactionQuery = 1;
			fa_PopulateDisplayTable();

			-- order the display
			fa_SortDisplayTable();
			-- display the data
			if (FactionAddictTab1:IsShown()) then
				FactionAddict_ScrollBar_Update();
			end;
			-- update info window if displayed
			if (frameFactionAddictInfoFrame:IsVisible()) then
				fa_InfoWindow_LoadInfo(frameFactionAddictInfoFrame);
			end;
		else -- not visible - just update flag to reload data
			doServerFactionQuery = 1;
		end;
	end
	
	-- Handle Bonus Faction Update 
	if (event == "LFG_BONUS_FACTION_ID_UPDATED") then
		fa_DebugOut("FactionAddict_OnEvent: " .. event);
		lfgBonusFactionID = GetLFGBonusFactionID();
		-- display the data
		if (frameFactionAddict:IsVisible() and FactionAddictTab1:IsShown()) then
			FactionAddict_ScrollBar_Update();
		end;
		-- update info window if displayed
		if (frameFactionAddictInfoFrame:IsVisible()) then
			fa_InfoWindow_LoadInfo(frameFactionAddictInfoFrame);
		end;

	end;

	if (event == "CHAT_MSG_COMBAT_FACTION_CHANGE") then
		fa_DebugOut("FactionAddict_OnEvent: " .. event);
		local factionDecrease = false;
		
		-- Check standing message for faction increase or decrease
		local s1, e1, faction, amount = string.find(arg1, pattern_standing_inc);
		-- If increase not found check for decrease
		if (s1 == nil) then
			s1, e1, faction, amount = string.find(arg1, pattern_standing_dec);
			factionDecrease = true;
		end;
		
		-- Seems to also be a case where this event will fire but not an UPDATE_FACTION
		-- which causes main tables to not get updated
		-- check if main window is displayed

		if (frameFactionAddict:IsVisible()) then
			-- update data for factions
			doServerFactionQuery = 1;
			fa_PopulateDisplayTable();

			-- order the display
			fa_SortDisplayTable();
			-- display the data
			if (FactionAddictTab1:IsShown()) then
				FactionAddict_ScrollBar_Update();
			end;
			-- update info window if displayed
			if (frameFactionAddictInfoFrame:IsVisible()) then
				fa_InfoWindow_LoadInfo(frameFactionAddictInfoFrame);
			end;
		else -- not visible - just update flag to reload data
			doServerFactionQuery = 1;
		end;


		-- workaround to get the guildname updated in faction data
		if (faction == GUILD and (faPlayerGuildName == GUILD or faPlayerGuildName == '' or faPlayerGuildName == nil)) then
			fa_PopulateAllFactionDataTable();
		end;
		
		-- Check that s1 is not nil before continuing
		if (s1 ~= nil and amount ~= nil) then
			local rep = tonumber(amount);
			if (factionDecrease) then
				rep = -rep;
			end;
			-- Override guild with actual guild name
			if (faction == GUILD) then
				faction = faPlayerGuildName;
			end;
			if (faction ~= nil) then
				fa_DebugOut ("Faction: " .. faction .. " Rep: " .. tostring(rep));
			end
			
			-- Check if factionID is nil - don't show as clickable text
			if (tempFactionIDsByName[faction] ~= nil) then 
				fa_DebugOut ("FactionID: " .. tostring(tempFactionIDsByName[faction]));
				if (rep > 0) then
					faLastRepGainStr = L.TAB2_LASTGAIN_TXT .. " - [" .. faction .. "]: " .. GREEN_FONT_COLOR_CODE .. tostring(rep) .. FONT_COLOR_CODE_CLOSE;
				else -- negative faction
					faLastRepGainStr = L.TAB2_LASTGAIN_TXT .. " - [" .. faction .. "]: " .. RED_FONT_COLOR_CODE .. tostring(rep) .. FONT_COLOR_CODE_CLOSE;
				end;

				-- auto-change rep bar
				if (FactionAddictConfig.CB_OPTION_AUTOBAR == true
					and faction ~= faPlayerGuildName ) then
						fa_SetWatchedFactionByID(tempFactionIDsByName[faction]);
				end;
			else
				fa_DebugOut ("FactionID: faction not in table!");
				if (rep > 0) then
					faLastRepGainStr = L.TAB2_LASTGAIN_TXT .. " - " .. faction .. ": " .. GREEN_FONT_COLOR_CODE .. tostring(rep) .. FONT_COLOR_CODE_CLOSE;
				else -- negative faction
					faLastRepGainStr = L.TAB2_LASTGAIN_TXT .. " - " .. faction .. ": " .. RED_FONT_COLOR_CODE .. tostring(rep) .. FONT_COLOR_CODE_CLOSE;
				end;
			end;
			
			fa_AddToRepLog(faction, rep);
			-- add to session total
			faSessionRepGain = faSessionRepGain + rep;
			FactionAddictTab2.faFactionsText2:SetText("Session Total: " .. HIGHLIGHT_FONT_COLOR_CODE .. tostring(faSessionRepGain) .. FONT_COLOR_CODE_CLOSE);
		end;
	end;
	
end;


--*****************************************************************************
--*****************************************************************************
function FactionAddict_ConfigMenuButton_OnClick(self)
	PlaySound("igInventoryRotateCharacter");
	ToggleDropDownMenu(1, nil, FactionAddict_ConfigMenu, self, 0, 0);
	fa_DebugOut("FactionAddict_ConfigMenuButton_OnClick");
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_ConfigMenu_OnLoad()
	local info =  {};
	local x;
    info.isTitle = 1;
	info.text = "Standing Filter";
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info, 1);
	
	for x = 1,9
	do
		info.text = ConfigMenuInfo[x].text;
		info.disabled = nil;
		info.isTitle = nil;
		info.func = fa_SetDDConfigValue;
		info.arg1 = ConfigMenuInfo[x].arg1;
		info.notCheckable = nil;
		info.checked = fa_GetDDConfigValue(ConfigMenuInfo[x].arg1);
		info.keepShownOnClick = 1;
		info.isNotRadio = 1;
		UIDropDownMenu_AddButton(info, 1);
	end;
	
    -- Add the above information to the options menu as a button.
	fa_DebugOut("FactionAddict_ConfigMenu_OnLoad");
end;

--*****************************************************************************
--*****************************************************************************
function FactionAddict_OnLoad(self)

	fa_DebugOut("FactionAddict_OnLoad: "..self:GetName());

	-- Register the frame so it can be dragged
	self:RegisterForDrag("LeftButton");
	
	-- Register Events
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("UPDATE_FACTION");
	self:RegisterEvent("LFG_BONUS_FACTION_ID_UPDATED");
	self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE");
	self:RegisterEvent("PLAYER_LOGIN");

	-- update portrait emblem
	SetPortraitToTexture(self.FactionAddictFramePortrait, "Interface\\ICONS\\achievement_reputation_01");
	
	-- Tab Handling code
	PanelTemplates_SetNumTabs(self, 4);
	PanelTemplates_SetTab(self, 1);
	
	-- show/hide panels
	FactionAddictTab1:Show();
	FactionAddictTab2:Hide();
	FactionAddictTab3:Hide();
	FactionAddictTab4:Hide();
	
	-- set addon text
	self.faTitleText:SetText(L.ADDON_TITLE);
	self.Tab1Label:SetText(L.TAB1_LABEL);
	self.Tab2Label:SetText(L.TAB2_LABEL);
	self.Tab3Label:SetText(L.TAB3_LABEL);
	self.Tab4Label:SetText(L.TAB4_LABEL);
	
	FactionAddictTab1.faDescText:SetText(L.TAB1_DESC_TXT);
	FactionAddictTab1.faErrorText1:Hide();
	FactionAddictTab2.faDescText:SetText(L.TAB2_DESC_TXT);
	FactionAddictTab2.hdr1:SetText(L.TAB2_LABEL);
	FactionAddictTab3.faDescText:SetText(L.TAB3_DESC_TXT);
	FactionAddictTab3.hdr1:SetText(L.TAB3_DISPLAY_CONFIG_TXT);
	FactionAddictTab3.hdr2:SetText(L.TAB3_DISPLAY_ORDER_TXT);
	FactionAddictTab3.hdr3:SetText(L.TAB3_DISPLAY_FILTER);
	FactionAddictTab3.hdr4:SetText(L.TAB3_DISPLAY_OPTIONS);
	CheckButton_CF.fs:SetText(L.TAB3_CONFIG_CF);
	CheckButton_BC.fs:SetText(L.TAB3_CONFIG_BC);
	CheckButton_WOTLK.fs:SetText(L.TAB3_CONFIG_WOTLK);
	CheckButton_CATA.fs:SetText(L.TAB3_CONFIG_CATA);
	CheckButton_MOP.fs:SetText(L.TAB3_CONFIG_MOP);
	CheckButton_WOD.fs:SetText(L.TAB3_CONFIG_WOD);
	CheckButton_OrderPct.fs:SetText(L.TAB3_CONFIG_ORDER_PCT);
	CheckButton_OrderName.fs:SetText(L.TAB3_CONFIG_ORDER_NAME);
	CheckButton_AO.fs:SetText(L.TAB3_CONFIG_ALLIANCE_ONLY);
	CheckButton_HO.fs:SetText(L.TAB3_CONFIG_HORDE_ONLY);
	CheckButton_AUTOBAR.fs:SetText(L.TAB3_CONFIG_AUTOBAR);
	CheckButton_HIDEINACTIVE.fs:SetText(L.TAB3_CONFIG_HIDEINACTIVE);
	FactionAddictTab4.faDescText:SetText(L.TAB4_DESC_TXT);
	FactionAddictTab4.faVersionText:SetText(L.VERSION_TXT..FACTION_ADDICT_VERSION);
	FactionAddictTab4.hdr1:SetText(L.TAB4_LABEL);
	FactionAddictAboutInfo:SetText(L.ABOUT_TXT);
	
	PanelTemplates_SetTab(frameFactionAddict, 1);
	
	-- init vars
	doServerFactionQuery = 1;
	faUpdateDisplayTable = true;

	faLastRepGainStr = L.TAB2_LASTGAIN_TXT .. " - " .. L.TAB2_NOGAIN_TXT;
	faSessionRepGain = 0;

	-- set date to use for this session
	local weekday = date("%A");
	local sessiondate = date("*t");
	faSessionDate = weekday .. " " .. sessiondate["month"] .. "/" .. sessiondate["day"];
	fa_DebugOut("faSessionDate: "..faSessionDate);
	
	FactionAddictTab2.faFactionsText2:SetText(L.TAB2_SESSIONTOTAL_TXT .. HIGHLIGHT_FONT_COLOR_CODE .. tostring(faSessionRepGain) .. FONT_COLOR_CODE_CLOSE);

	-- make closable with escape
	tinsert(UISpecialFrames,self:GetName());
end;